@@ -4,16 +4,22 @@ import os
 
 __author__ = """Michael Watson"""
 __email__ = 'mike.watson@sheffield.ac.uk'
-__version__ = '0.1.1'
+__version__ = '0.2.0'
 
 try:
     import cupy  # noqa: F401
     CUDA = True
+    asnumpy = cupy.asnumpy
 except ImportError:
     CUDA = False
+    import numpy
+    asnumpy = numpy.asarray
 
 CORES = multiprocessing.cpu_count()
 OUTPUT_DIR = os.getcwd()
+ERROR_IF_MISSING_MODEL = True
+ERROR_IF_MISSING_SUB_MODEL = True
+ERROR_IN_DATA_CHECK = True
 
 
 class OverRideCuda:
